<?php

use App\Domain\Categories\Models\Category;
use App\Domain\Categories\Models\Property;
use App\Domain\Categories\Models\PropertyDirectoryValue;
use App\Domain\Classifiers\Models\Brand;
use App\Domain\Kafka\Messages\Send\ModelEvent\BrandPayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\CategoryPayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\PropertyDirectoryValuePayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\PropertyPayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\PublishedImagePayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\PublishedProductPayload;
use App\Domain\Kafka\Messages\Send\ModelEvent\PublishedPropertyValuePayload;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Domain\Products\Models\ProductPropertyValue;
use App\Domain\Products\Models\PublishedProduct;
use Tests\IntegrationTestCase;
use function PHPUnit\Framework\assertEquals;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-send-messages');

test("generate BrandPayload success", function () {
    $model = Brand::factory()->create();

    $payload = new BrandPayload($model);

    assertEquals($payload->jsonSerialize(), $model->attributesToArray());
});
test("generate CategoryPayload success", function () {
    $model = Category::factory()->create();

    $payload = new CategoryPayload($model);

    assertEquals($payload->jsonSerialize(), $model->attributesToArray());
});
test("generate PublishedImagePayload success", function () {
    $model = ProductImage::factory()->published()->create();

    $payload = new PublishedImagePayload($model->release);

    assertEquals($payload->jsonSerialize(), $model->release->attributesToArray());
});
test("generate PublishedProductPayload success", function () {
    $product = Product::factory()->create();
    /** @var PublishedProduct $model */
    $model = PublishedProduct::query()->find($product->id);

    $payload = new PublishedProductPayload($model);

    assertEquals($payload->jsonSerialize(), $model->attributesToArray());
});
test("generate PublishedPropertyValuePayload success", function () {
    $model = ProductPropertyValue::factory()->published()->create();

    $payload = new PublishedPropertyValuePayload($model->release);

    assertEquals($payload->jsonSerialize(), $model->release->attributesToArray());
});
test("generate PropertyPayload success", function () {
    $model = Property::factory()->create();

    $payload = new PropertyPayload($model);

    assertEquals($payload->jsonSerialize(), $model->attributesToArray());
});
test("generate PropertyDirectoryValuePayload success", function () {
    $model = PropertyDirectoryValue::factory()->create();

    $payload = new PropertyDirectoryValuePayload($model);

    assertEquals($payload->jsonSerialize(), $model->attributesToArray());
});
